
import numpy as np
from .util import UKN

import pandas as pd
from sklearn.metrics import roc_auc_score
from .list_datapoints import ListofDatapoints
from .datapoint import Datapoint    

def filter(listofdatapoints:ListofDatapoints, min_AUROC: int = 0.8):
        """Filters out duplicate sequences.
        Only keep the first occurence of a sequence if all the other structures are the same.

        Examples:
            >>> datapoints = ListofDatapoints([ Datapoint(reference='ref1', sequence='AACCGG', structure=[[1, 2], [3, 4]], dms=[1,0,0,0,0,1]),\
                                Datapoint(reference='ref2', sequence='AACCGG', structure=[[1, 2], [3, 4]], dms=[1,0,0,0,0,1]),\
                                Datapoint(reference='ref2', sequence='AACCGG', structure=[[1, 2], [3, 4]], dms=[1,0,0,0,0,1]),\
                                Datapoint(reference='ref4', sequence='AUGGC', structure=[[1, 2]], dms=[0,0,0,0,1]),\
                                Datapoint(reference='ref5', sequence='AUGGC', structure=[[0, 4], [2, 3]], dms=[0,0,0,0,0]),\
                                Datapoint(reference='ref6', sequence='not a regular sequence', structure=[[0, 4]], dms=[0,0,0,0,0]) ])
            >>> print(filter(datapoints, min_AUROC=0))
            Over a total of 6 datapoints, there are:
            ### OUTPUT
            - 3 valid datapoints
            ### MODIFIED
            - 1 multiple sequences with the same reference (renamed reference)
            - 1 duplicate sequences with different structure / dms / shape
            ### FILTERED OUT
            - 1 invalid datapoints (ex: sequence with non-regular characters)
            - 0 datapoints with bad structures
            - 2 duplicate sequences with the same structure / dms / shape
            - 0 datapoints removed because of low AUROC (<0)
        """

        # Remove None datapoints
        n_input_datapoints = len(listofdatapoints)
        datapoints, n_unvalid_datapoints = listofdatapoints.drop_none_dp()

        # Remove bad structures
        bad_structures_idx = []
        for idx, datapoint in enumerate(datapoints):
            if not datapoint._assert_structure():
                bad_structures_idx.append(idx)
        datapoints = [
            datapoint
            for idx, datapoint in enumerate(datapoints)
            if idx not in bad_structures_idx
        ]
        n_bad_structures_datapoints = len(bad_structures_idx)

        # Convert to pandas
        df = listofdatapoints.to_pandas(datapoints)

        # Remove duplicate or conflicting datapoints and keep track of the number of datapoints removed
        def drop_duplicates(df: pd.DataFrame, **kwargs):
            len_df_before = len(df)
            df.drop_duplicates(**kwargs)
            return len_df_before - len(df)

        # If multiple sequences with the same reference, rename the reference
        refs = dict()
        n_same_ref_datapoints = 0
        for idx, row in df.iterrows():
            if row["reference"] in refs:
                df.at[idx, "reference"] = f"{row['reference']}_{refs[row['reference']]}"
                n_same_ref_datapoints += 1
            refs[row["reference"]] = refs.get(row["reference"], 0) + 1

        # Keep only one datapoint per sequence and structure
        if "structure" in df.columns:
            n_duplicates_datapoints = drop_duplicates(
                df,
                subset=["sequence", "structure"],
                inplace=True,
                ignore_index=True,
                keep="first",
            )

        # Keep only one datapoint per sequence and signal
        for signal in ["dms", "shape"]:
            if signal in df.columns:
                if not "structure" in df.columns:
                    n_duplicates_datapoints = 0
                n_duplicates_datapoints += drop_duplicates(
                    df,
                    subset=["sequence", signal],
                    inplace=True,
                    ignore_index=True,
                    keep="first",
                )

        # Count how many multiple structures / dms with the same sequence
        n_same_seq_datapoints = 0
        for _, group in df.groupby("sequence"):
            if len(group) > 1:
                n_same_seq_datapoints += len(group) - 1
            
        ## Filter out references with low AUROC
        mask_high_AUROC = None
        if "structure" in df.columns and "dms" in df.columns or "shape" in df.columns:
            
            signal = "dms" if "dms" in df.columns else "shape"
            def calculate_auroc(row):
                sig = np.array(row[signal])
                isUnpaired = np.ones_like(sig)
                isUnpaired[np.array(row["structure"]).flatten()] = 0
                if set(isUnpaired[sig != UKN]) != set([0, 1]):
                    return 0
                return roc_auc_score(isUnpaired[sig != UKN], sig[sig != UKN])

            # Create a boolean mask for rows with auroc score greater than or equal to a threshold
            mask_high_AUROC = df.apply(
                lambda row: calculate_auroc(row) >= min_AUROC, axis=1
            )
            df = df[mask_high_AUROC]

        # Convert back to list of datapoints
        listofdatapoints.datapoints = listofdatapoints.from_pandas(df)

        # Write report
        report = f"""Over a total of {n_input_datapoints} datapoints, there are:
### OUTPUT
- {len(listofdatapoints)} valid datapoints
### MODIFIED
- {n_same_ref_datapoints} multiple sequences with the same reference (renamed reference)
- {n_same_seq_datapoints} duplicate sequences with different structure / dms / shape
### FILTERED OUT
- {n_unvalid_datapoints} invalid datapoints (ex: sequence with non-regular characters)
- {n_bad_structures_datapoints} datapoints with bad structures"""
        if "structure" in df.columns or "dms" in df.columns or "shape" in df.columns:
            report += f"""
- {n_duplicates_datapoints} duplicate sequences with the same structure / dms / shape"""
        if mask_high_AUROC is not None:
            report += f"""
- {np.sum(~mask_high_AUROC)} datapoints removed because of low AUROC (<{min_AUROC})"""

        return report


import os

# Define the one-hot encodings for the sequences and structures
seq2int = {
    "X": 0,
    "A": 1,
    "C": 2,
    "G": 3,
    "U": 4,
}

int2seq = {v: k for k, v in seq2int.items()}

dot2int = {".": 1, "(": 2, ")": 3, "X": 0}
int2dot = {v: k for k, v in dot2int.items()}

UKN = -1000


class DreemUtils:
    def flatten_json(data):
        out, row = [], {}

        for k, v in data.copy().items():
            if type(v) != dict:
                row[k] = v
            else:
                row["reference"] = k
                for k2, v2 in v.items():
                    if type(v2) != dict:
                        row[k2] = v2
                    else:
                        row["section"] = k2
                        for k3, v3 in v2.items():
                            row["cluster"] = k3
                            if type(v3) != dict:
                                row[k3] = v3
                            else:
                                for k4, v4 in v3.items():
                                    row[k4] = v4
                                out.append(row.copy())
        return out

    def sort_dict(mut_profiles):
        sorting_key = lambda item: 1000 * {int: 0, str: 0, float: 0, list: 1, dict: 2}[
            type(item[1])
        ] + ord(item[0][0])
        mut_profiles = {k: mut_profiles[k] for k in sorted(mut_profiles)}
        mut_profiles = dict(sorted(mut_profiles.items(), key=sorting_key))
        for k, v in mut_profiles.items():
            if type(v) is not dict:
                continue
            mut_profiles[k] = DreemUtils.sort_dict(v)
        return mut_profiles


def standardize_sequence(sequence):
    sequence = sequence.upper()
    sequence = sequence.replace("T", "U")
    # trim the sequence
    sequence = sequence.replace(" ", "").replace("\n", "").replace("\t", "")
    return sequence


def sequence_has_regular_characters(sequence):
    return not (set(sequence) - set("ACGU"))



from os.path import join
import os
from .env import Env


class Path:
    """Path to files and folders of a datafolder of name `name`. The path to the data folder is `DATA_FOLDER`, which is defined in `env`.

    Parameters
    ----------

    name : str

    Returns
    -------

    Path

    Example
    -------

    >>> path = Path(name='my_test_datafolder_pytest', root='data')
    >>> path.name
    'my_test_datafolder_pytest'
    >>> print(path)
    Path(name='my_test_datafolder_pytest')
    """

    def __init__(self, name, root) -> None:
        assert isinstance(name, str), f"name {name} is not a string"
        assert isinstance(root, str), f"root {root} is not a string"
        self.root = root
        self.name = name

    def make(self, force=False):
        """Creates the data folder."""
        if force:
            self.clear()
        os.system(f"mkdir -p {self.get_main_folder()}")

    def clear(self):
        """Clears the data folder."""
        os.system(f"rm -rf {self.get_main_folder()}")

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(name='{self.name}')"

    def get_data_folder(self) -> str:
        """Returns the path to the data folder."""
        return self.root

    def get_main_folder(self) -> str:
        """Returns the path to the main folder."""
        return join(self.get_data_folder(), self.name)

    def get_data_json(self) -> str:
        """Returns the path to the data.json file."""
        return join(self.get_main_folder(), "data.json")

    def get_card(self) -> str:
        """Returns the path to the README.md file."""
        return join(self.get_main_folder(), "README.md")

    def get_conversion_report(self) -> str:
        """Returns the path to the conversion report file."""
        return join(self.get_main_folder(), "conversion_report.txt")



from .list_datapoints import ListofDatapoints
from .path import Path
from .filter import filter as filter_datapoints

def convert(
    format: str,
    file_or_folder: str,
    name: str = None,
    path_out: str = "data",
    predict_structure: bool = False,
    filter: bool = True,
    min_AUROC=0.8,
    verbose: bool = True,
):
    """Converts a file or folder into a json file. Different formats are supported.
    
    Args:
        format (str): Format of the input file. Can be 'ct', 'seismic', 'json', 'bpseq' or 'fasta'.
        file_or_folder (str): Path to the file or folder to convert.
        name (str, optional): Name of the dataset. Defaults to None, in which case the name of the file or folder will be used.
        path_out (str, optional): Path to the output folder. Defaults to 'data'.
        predict_structure (bool, optional): Whether to predict the structure or not using RNAstructure. Defaults to False.
        filter (bool, optional): Whether to filter the datapoints or not. Defaults to True. Datapoints with no sequence or reference will be dropped anyways.
        min_AUROC (float, optional): Minimum AUROC to keep a datapoint. Defaults to 0.8.
        verbose (bool, optional): Whether to print the conversion report or not. Defaults to True.
    """
    assert format in ["ct", "seismic", "json", "bpseq", "fasta"], "Format not supported"

    if name is None:
        name = file_or_folder.split("/")[-1].split(".")[0]
    path = Path(name=name, root=path_out)
    path.make()

    if format == "ct":
        datapoints = ListofDatapoints.from_ct(
            file_or_folder, tqdm=True, verbose=verbose
        )

    elif format == "seismic":
        datapoints = ListofDatapoints.from_dreem_output(
            file_or_folder, predict_structure, tqdm=True, verbose=verbose
        )

    elif format == "json":
        datapoints = ListofDatapoints.from_json(
            file_or_folder, predict_structure, tqdm=True, verbose=verbose
        )

    elif format == "bpseq":
        datapoints = ListofDatapoints.from_bpseq(
            file_or_folder, tqdm=True, verbose=verbose
        )

    elif format == "fasta":
        datapoints = ListofDatapoints.from_fasta(
            file_or_folder, predict_structure, tqdm=True, verbose=verbose
        )

    if filter:
        report = filter_datapoints(datapoints, min_AUROC=min_AUROC)
    else:
        _, report = datapoints.drop_none_dp()
        report = (
            f"Drop {report} datapoints with None values (null sequence or reference)"
        )

    if verbose:
        print(report)

    with open(path.get_conversion_report(), "w") as f:
        f.write("# Conversion report \n\n")
        f.write(report)

    if path_out is not None:
        datapoints.to_json(path.get_data_json())

    return datapoints.to_dict()



from huggingface_hub import snapshot_download
import os
from os.path import dirname, exists
import json
from huggingface_hub import HfApi
import datetime
import numpy as np
import json

from .path import Path
from .env import Env


def get_dataset(name: str, force_download=False, tqdm=True):
    """Get a dataset from HuggingFace or from the local cache.

    Args:
        name (str): Name of the dataset.
        force_download (bool, optional): Whether to force the download or not. Defaults to False.
        tqdm (bool, optional): Whether to display a progress bar or not. Defaults to True.
    """

    path = Path(name=name)

    if force_download:
        os.system(f"rm -rf {path.get_main_folder()}")

    if not exists(path.get_data_json()):
        print("{}: Downloading dataset from HuggingFace Hub...".format(name))
        download_dataset(name)
        print(
            "{}: Download complete. File saved at {}".format(name, path.get_data_json())
        )

    return json.load(open(path.get_data_json(), "r"))


def download_dataset(name: str):
    """Download a dataset from HuggingFace Hub. The name corresponds to the name of the dataset on HuggingFace Hub."""
    snapshot_download(
        repo_id="rouskinlab/" + name,
        repo_type="dataset",
        local_dir=Path(name).get_main_folder(),
        token=Env.get_hf_token(),
        allow_patterns=["data.json"],
    )


def name_from_path(datapath: str):
    if datapath.split("/")[-1] == "data.json":
        return datapath.split("/")[-2]
    return datapath.split("/")[-1].split(".")[0]


def clean_data(datapath: str):
    data = json.load(open(datapath, "r"))
    for ref, values in data.items():
        copy = values.copy()
        for k, v in values.items():
            if v is None or (isinstance(v, float) and np.isnan(v)):
                copy.pop(k)
        data[ref] = copy
    return data


def upload_dataset(
    datapath: str, exist_ok=False, commit_message: str = None, add_card=True, **kwargs
):
    api = HfApi()
    name = name_from_path(datapath)
    # data = clean_data(datapath)

    hf_token = Env.get_hf_token()

    api.create_repo(
        repo_id="rouskinlab/" + name,
        token=hf_token,
        exist_ok=exist_ok,
        private=True,
        repo_type="dataset",
    )

    api.upload_file(
        path_or_fileobj=datapath,
        path_in_repo="data.json",
        repo_id="rouskinlab/" + name,
        repo_type="dataset",
        token=hf_token,
        commit_message=commit_message,
        **kwargs,
    )

    if add_card:
        card = write_card(datapath)
        api.upload_file(
            path_or_fileobj=card,
            repo_id="rouskinlab/" + name,
            path_in_repo="README.md",
            repo_type="dataset",
            token=hf_token,
            commit_message=commit_message,
        )


def write_card(datapath):
    source = os.path.basename(datapath)
    date = datetime.datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
    name = name_from_path(datapath)

    out = """
---
license: mit
language:
  - en
tags:
  - chemistry
  - biology`
author: Silvi Rouskin
source: {}
date: {}
---


# Data types
""".format(
        source, date
    )

    data_type_count = {}
    data = json.load(open(datapath, "r"))
    for dp in data.values():
        for k, v in dp.items():
            if v is None or (isinstance(v, float) and np.isnan(v)):
                continue
            if k not in data_type_count:
                data_type_count[k] = 0
            data_type_count[k] += 1

    for k, v in data_type_count.items():
        out += f"""
- **{k}**: {v} datapoints"""

    # TODO add filtering report

    # dump
    path = Path(name=name, root=dirname(dirname(datapath)))
    os.makedirs(os.path.dirname(path.get_card()), exist_ok=True)
    with open(path.get_card(), "w") as f:
        f.write(out)

    if os.path.exists(path.get_conversion_report()):
        conversion = open(path.get_conversion_report(), "r").read()
        with open(path.get_card(), "a") as f:
            f.write("\n\n" + conversion)

    return path.get_card()



import os
import numpy as np
from .datapoint import Datapoint, DatapointFactory
from typing import List
from .parsers import Fasta, DreemOutput

import pandas as pd
from tqdm import tqdm as tqdm_parser
import json


class ListofDatapoints:
    """Class to store a list of datapoints."""
    def __init__(self, datapoints=[], verbose=True):
        self.datapoints = datapoints

    def __call__(self) -> List[Datapoint]:
        return self.datapoints
    
    def __len__(self) -> int:
        return len(self.datapoints)

    @classmethod
    def from_fasta(
        cls, fasta_file, predict_structure, tqdm=True, verbose=True
    ) -> "ListofDatapoints":
        """Create a list of datapoint from a fasta file. The structure and dms will be predicted if predict_structure and predict_dms are True."""
        sequences, references = Fasta.parse(fasta_file)
        return cls(
            [
                DatapointFactory.from_fasta(sequence, reference, predict_structure)
                for sequence, reference in tqdm_parser(
                    zip(sequences, references),
                    total=len(sequences),
                    desc="Parsing fasta file",
                    disable=not tqdm,
                )
            ],
            verbose=verbose,
        )

    @classmethod
    def from_bpseq(cls, bpseq_files, tqdm=True, verbose=True):
        """Create a list of datapoint from a bpseq file. The dms will be predicted if predict_dms is True."""
        return cls(
            [
                DatapointFactory.from_bpseq(ct_file)
                for ct_file in tqdm_parser(
                    bpseq_files,
                    total=len(bpseq_files),
                    desc="Parsing bpseq files",
                    disable=not tqdm,
                )
            ],
            verbose=verbose,
        )

    @classmethod
    def from_ct(cls, ct_folder, tqdm=True, verbose=True):
        """Create a list of datapoint from a ct file. The dms will be predicted if predict_dms is True."""
        ct_files = [
            os.path.join(ct_folder, ct_file)
            for ct_file in os.listdir(ct_folder)
            if ct_file.endswith(".ct")
        ]
        return cls(
            [
                DatapointFactory.from_ct(ct_file)
                for ct_file in tqdm_parser(
                    ct_files,
                    total=len(ct_files),
                    desc="Parsing ct files",
                    disable=not tqdm,
                )
            ],
            verbose=verbose,
        )

    @classmethod
    def from_dreem_output(
        cls, dreem_output_file, predict_structure, tqdm=True, verbose=True
    ):
        """Create a list of datapoint from a dreem output file. The structure and dms will be predicted if predict_structure and predict_dms are True."""
        n_lines = len(list(DreemOutput.parse(dreem_output_file)))
        return cls(
            [
                DatapointFactory.from_dreem_output(
                    reference, sequence, mutation_rate, predict_structure
                )
                for reference, sequence, mutation_rate in tqdm_parser(
                    DreemOutput.parse(dreem_output_file),
                    total=n_lines,
                    desc="Parsing dreem output file",
                    disable=not tqdm,
                )
            ],
            verbose=verbose,
        )

    @classmethod
    def from_json(cls, json_file, predict_structure=False, tqdm=True, verbose=True):
        """Create a list of datapoint from a json file."""
        data = json.load(open(json_file))
        return cls(
            [
                DatapointFactory.from_json_line(reference, line, predict_structure)
                for reference, line in tqdm_parser(
                    data.items(),
                    total=len(data),
                    desc="Parsing json file",
                    disable=not tqdm,
                )
            ],
            verbose=verbose,
        )

    def to_dict(self) -> dict:
        """Converts the list of datapoints into a dictionary."""
        return {
            datapoint.reference: datapoint.to_dict()[1] for datapoint in self.datapoints
        }

    def to_json(self, path) -> None:
        """Write a list of datapoints to a json file."""
        if os.path.dirname(path) != "":
            os.makedirs(os.path.dirname(path), exist_ok=True)
        with open(path, "w") as f:
            f.write("{\n")
            for idx, datapoint in enumerate(
                self.datapoints
            ):  # write the datapoints one by one to avoid memory issues
                f.write(
                    str(datapoint) + ",\n"
                    if idx != len(self.datapoints) - 1
                    else str(datapoint) + "\n"
                )
            f.write("}\n")

    def to_pandas(self, datapoints=None) -> pd.DataFrame:
        """Converts the list of datapoints into a pandas dataframe.

        Example:
        >>> datapoints = ListofDatapoints([Datapoint(reference='reference', sequence='AACCGG', structure=[[1, 2], [3, 4]], dms=[1,0,0,0,0,1])], verbose=False)
        >>> temp = datapoints.to_pandas()
        >>> temp['reference'].iloc[0]
        'reference'
        >>> temp['sequence'].iloc[0]
        'AACCGG'
        >>> temp['structure'].iloc[0]
        ((1, 2), (3, 4))
        >>> temp['dms'].iloc[0]
        (1, 0, 0, 0, 0, 1)
        """
        if datapoints is None:
            datapoints = self.datapoints
        for dp in datapoints:
            dp.convert_arrays_to_tuple()
        return pd.DataFrame([datapoint.to_flat_dict() for datapoint in datapoints])

    def from_pandas(self, df: pd.DataFrame) -> None:
        """Converts a pandas dataframe into a list of datapoints.

        Example:
            >>> df = pd.DataFrame([{'reference': 'reference', 'sequence': 'AACCGG', 'structure': [[1, 2], [3, 4]], 'dms': [1.0, 2.0, 3.0]}])
            >>> ListofDatapoints(verbose=False).from_pandas(df)
            [Datapoint('reference', sequence='AACCGG', structure=[[1, 2], [3, 4]], dms=[1.0, 2.0, 3.0])]
        """
        datapoints = [
            Datapoint.from_flat_dict(datapoint_dict)
            for datapoint_dict in df.to_dict("records")
        ]
        for dp in datapoints:
            dp.convert_arrays_to_list()
        return datapoints

    def drop_none_dp(self):
        n_input_datapoints = len(self.datapoints)
        self.datapoints = list(filter(lambda x: x is not None, self.datapoints))
        n_unvalid_datapoints = n_input_datapoints - len(self.datapoints)
        return self.datapoints, n_unvalid_datapoints

    



import os
import numpy as np
import pandas as pd
from .env import Env


def run_command(cmd):
    import subprocess

    process = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE)
    output, error = process.communicate()
    return output.decode("utf-8")


class RNAstructure(object):

    """RNAstructure wrapper."""

    def __init__(self) -> None:
        pass

    def predict_partition(self, temperature_k=None, dms=None):
        # predict the partition of rna structures
        cmd = f"{os.path.join(Env.get_rnastructure_path(), 'partition')} {self.fasta_file} {self.pfs_file}"
        if temperature_k != None:
            cmd += " --temperature " + str(temperature_k)
        if type(dms) != type(None):
            assert len(self.sequence) == len(
                dms
            ), "The length of the sequence is not the same as the length of the signal."
            assert type(dms) in [
                list,
                tuple,
                np.ndarray,
            ], "The dms signal should be a list of floats."
            self.__write_dms_to_file(self.sequence, dms)
            cmd += " --shape " + self.dms_file
        run_command(cmd)

        # sum it into pairing probability
        run_command(
            os.path.join(Env.get_rnastructure_path(), "ProbabilityPlot")
            + " "
            + self.pfs_file
            + " -t "
            + self.prob_file
        )

        # read the probability file
        with open(self.prob_file, "r") as f:
            lines = f.readlines()
            pairingPrediction = {"i": [], "j": [], "p": []}
            for x in range(len(lines)):
                if x > 1:
                    ls = lines[x].split("\t")
                    pairingPrediction["i"] += [int(ls[0])]
                    pairingPrediction["j"] += [int(ls[1])]
                    pairingPrediction["p"] += [float(ls[2])]
        df = pd.DataFrame(pairingPrediction)

        # turn log probability into probability
        df["p"] = df["p"].apply(lambda x: np.power(10, -x))

        # add the reverse complement so that when we sum, we get the probability of each base being paired with anyone
        df = pd.concat([df, df.rename(columns={"i": "j", "j": "i"})])

        # cast: two bases being paired together => probability for each base of being paired with anyone
        df = df.groupby(["i"]).sum().drop(columns=["j"])

        # add p=0 for bases that are not paired with anyone
        df = df.reindex(range(1, len(self.sequence) + 1), fill_value=0)

        # Logs sum can lead to a probability > 1, but it should never be > 1.05. So we'll sanity check that, then cap it at 1.
        assert (df["p"] >= 0).all() and (
            df["p"] <= 1.05
        ).all(), "The probability is not between 0 and 1.05, something is wrong. Check the log sum."
        df["p"] = df["p"].apply(lambda x: min(x, 1))

        # Make sure all the bases are here
        assert len(df) == len(
            self.sequence
        ), "The number of bases in the sequence is not the same as the number of bases in the prediction."

        return df["p"].tolist()

    def __write_dms_to_file(self, sequence, signal):
        assert len(sequence) == len(
            signal
        ), "The length of the sequence is not the same as the length of the signal."

        with open(self.dms_file, "w") as f:
            for idx, (s, b) in enumerate(zip(signal, sequence)):
                if b in "AC":
                    f.write(f"{idx+1}\t{s}\n")

    def __make_temp_folder(self):
        """Remove content and make a new folder for temporary files."""
        path = Env.get_rnastructure_temp_path()
        if os.path.exists(path):
            import shutil

            shutil.rmtree(path)
        os.makedirs(path)

    def __make_files(self, temp_prefix="temp"):
        this_file = lambda x: os.path.join(Env.get_rnastructure_temp_path(), x)
        self.pfs_file = this_file(f"{temp_prefix}.pfs")
        self.ct_file = this_file(f"{temp_prefix}.ct")
        self.dms_file = this_file(f"{temp_prefix}.shape")
        self.dot_file = this_file(f"{temp_prefix}_dot.txt")
        self.fasta_file = this_file(f"{temp_prefix}.fasta")
        self.prob_file = this_file(f"{temp_prefix}_prob.txt")

    def __create_fasta_file(self, reference, sequence):
        # push the ref into a temp file
        temp_fasta = open(self.fasta_file, "w")
        temp_fasta.write(">" + reference + "\n" + sequence)
        temp_fasta.close()

    def predictPairingProbability(self, sequence, dms=None, reference="reference"):
        self.sequence = sequence
        self.__make_temp_folder()
        self.__make_files()
        self.__create_fasta_file(reference, sequence)
        return self.predict_partition(dms=dms)

    def predictStructure(self, sequence, dms=None, shape=None):
        self.sequence = sequence
        self.__make_temp_folder()
        self.__make_files()
        self.__create_fasta_file("reference", sequence)
        envir = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
        import sys

        files = sys.path
        cmd = f"{os.path.join(Env.get_rnastructure_path(), 'Fold')} {self.fasta_file} {self.ct_file}"
        if type(dms) != type(None):
            assert len(sequence) == len(
                dms
            ), "The length of the sequence is not the same as the length of the signal."
            assert type(dms) in [
                list,
                tuple,
                np.ndarray,
            ], "The dms signal should be a list of floats."
            self.__write_dms_to_file(sequence, dms)
            cmd += " --dms " + self.dms_file
        if type(shape) != type(None):
            assert len(sequence) == len(
                shape
            ), "The length of the sequence is not the same as the length of the signal."
            assert type(shape) in [
                list,
                tuple,
                np.ndarray,
            ], "The shape signal should be a list of floats."
            self.__write_dms_to_file(sequence, shape)
            cmd += " --shape " + self.dms_file
        run_command(cmd)
        cmd = f"{os.path.join(Env.get_rnastructure_path(), 'ct2dot')} {self.ct_file} 0 {self.dot_file}"
        run_command(cmd)
        assert os.path.exists(
            self.dot_file
        ), 'The dot file was not created. Check RNAstructure installation. If you use a Mac, make sure to run `setup_env(RNASTRUCTURE_PATH="abs/path/to/RNAstructure/exe")`.'
        with open(self.dot_file, "r") as f:
            return f.readlines()[2].strip()


RNAstructure_singleton = RNAstructure()
